#ifndef __INTERFACE_H_
 #define __INTERFACE_H_

/*
   Interface flags
*/
#define INTERFACE_UNKNOWN                        0
#define INTERFACE_NONE           INTERFACE_UNKNOWN /* No interface */
#define INTERFACE_STREAM                         1 /* Stream       */
#define INTERFACE_BUFFER                         2 /* Buffer       */
#define INTERFACE_BUFFER_PEICE                   4 /* Peice buffer */
#define INTERFACE_ASYNC							 8 /* Asynchronous */

#endif
