#ifndef __BUFFER_H_
 #define __BUFFER_H_

#include "line.h"

typedef struct __buffer_ {
  int fd;
  line_t *l;
  unsigned long cin, blen, ulen, clen;
  char *b, restart, fn, eof, fbrk, *bp, *lbp;
} buffer_t;

buffer_t *buffer_init(int, unsigned long);
void buffer_restart(buffer_t *, int);
void buffer_kill(buffer_t *);
char *buffer_next_line(buffer_t *);

#endif
