#ifndef __ADDRESS_H_
 #define __ADDRESS_H_

typedef struct __address_ {
  char *name, *user,
       *domain;

  struct __address_ *next;
} address_t;

typedef struct __group_ {
  char *group;
  unsigned long nmembers;
  struct __address_ *members;
} group_t;

group_t *address_evaluate(const char *);
address_t *address_evaluate_one(char *);
void address_kill(group_t *);
void address_kill_one(address_t *);

#endif
