#include <stdio.h>
#include "eps.h"

int int_stream_init(eps_t *e, int *args)
{
  e->u = unfold_init(*args, MAX_LINE_LENGTH);
  if (e->u == NULL)
     return 0;

  return 1;
}

void int_stream_restart(eps_t *e, int *args)
{
  unfold_restart(e->u, *args);
}
