#ifndef __RFC2822_H_
 #define __RFC2822_H_

#define MAX_LINE_LENGTH 1000

inline int rfc2822_is_wsp(unsigned char);
inline int rfc2822_is_sep(unsigned char);
unsigned char *rfc2822_remove_comments(unsigned char *);
unsigned char *rfc2822_next_token(unsigned char *, unsigned char, unsigned char *);
unsigned char *rfc2822_convert_literals(unsigned char *);

#endif
