#ifndef __BOUNDARY_H_
 #define __BOUNDARY_H_

typedef struct __member_ {
  char *boundary,
       depth;

  struct __member_ *next;
} member_t;

typedef struct __boundary_ {
  char cdepth;
  struct __member_ *boundaries, *last;
} boundary_t;

boundary_t *boundary_alloc(void);
int boundary_add(eps_t *, char *);
void boundary_kill(boundary_t *);
int boundary_is(eps_t *, char *);
int boundary_remove_last(eps_t *);
char *boundary_fetch(eps_t *, char);
void boundary_debug(eps_t *);

#endif
